import requests

from . import idec
from . import idict


@idec.Retry('InitAPI', cycle=10)
def __init__(name: str, conf: idict):
    return IsddcAPI(
        api=conf.api,
        keystone=conf.keystone,
        username=conf.username,
        password=conf.password,
        tenantid=conf.tenantid,
    )


class IsddcAPI:
    headers = {
        'Content-Type': 'application/json',
        'X-Auth-Token': None
    }

    uri = idict(
        metrics='/v2.0/metrics',
        inventories='/v2.0/inventories'
    )

    def __init__(
            self,
            api: str = 'isddc-api:30870',
            keystone: str = 'isddc-keystone:31500',
            username: str = '_data_collector',
            password: str = 'P@ssw0rd!',
            tenantid: str = '156bae0eb6ed4916a88bfc56284fa3cd',
            timeout: int = 30
    ):
        self.api = api
        self.keystone = keystone
        self.username = username
        self.password = password
        self.tenantid = tenantid
        self.timeout = timeout

        self.auth_api = f'http://{self.keystone}/v3/auth/tokens'

        for name, uri in self.uri.items():
            self.uri[name] = f'http://{self.api}{uri}'

    def __getattribute__(self, name):
        if name in ['send']:
            self.headers['X-Auth-Token'] = self._x_auth_token

        return super().__getattribute__(name)

    @property
    def _x_auth_token(self) -> str:
        return requests.post(
            url=self.auth_api,
            json={'auth': {
                'identity': {
                    'methods': ['password'],
                    'password': {'user': {
                        'domain': {'id': 'default'},
                        'name': self.username,
                        'password': self.password}}},
                'scope': {'project': {
                    'id': self.tenantid,
                    'domain': {'id': 'default'}}}}}
        ).headers['X-Subject-Token']

    def send(
            self,
            data: dict,
            dtype: 'enum(metrics, inventories)'
    ) -> requests.Response:
        return requests.post(
            url=self.uri[dtype],
            json=data,
            headers=self.headers,
            timeout=self.timeout
        )


oi: IsddcAPI
