import json
import threading as th

from pykafka import Topic
from pykafka import Producer
from pykafka import KafkaClient

from . import idec
from . import idict

__default__: Topic


@idec.Retry('InitKafka', cycle=10)
def __init__(name: str, conf: idict) -> Topic:
    client = KafkaClient(**conf)
    topic: Topic = client.topics[name]
    return topic


def send(msg: dict, topic: Topic = None):
    """Simple transmitter."""
    pro: Producer = (topic or __default__).get_sync_producer()
    pro.produce(json.dumps(msg).encode())

    # `pro.stop()` is very slow, so:
    th.Thread(
        target=lambda: pro.stop(),
        daemon=True,
        name='StopProducer'
    ).start()
